/*
 * http://www.kerrywong.com
 * Dual tracking power supply
 * 12/2013
 */

#include <LiquidCrystal.h>
#include <SPI.h>
#include <Encoder.h>

const int pinRS = 2;
const int pinEn = 4;
const int pinD4 = 17;
const int pinD5 = 16;
const int pinD6 = 15;
const int pinD7 = 14;

const int pinBtnEncV = 7;
const int pinBtnEncC = 8;

const int pinEncV1 = 3;
const int pinEncV2 = 5;
const int pinEncI1 = 6;
const int pinEncI2 = 9;

const int pinCSV = 10;
const int pinCSI = 18;

const int LOOP_MAX_COUNT = 1000;

int buttonPins[]={
    pinBtnEncV, pinBtnEncC};
int buttonReadings[1];
int lastButtonStates[]={
    HIGH, HIGH};
int currentButtonStates[]={
    HIGH, HIGH};
long lastDebounceTime[1];

long oldEncPositions[] = {
    -999, -999};
long curEncPositions[] = {
    0, 0};
long encoderValues[] = {
    0, 0};
int DACSetSteps[] = {
    1, 1};

int loopCounter = 0;

LiquidCrystal lcd(pinRS, pinEn, pinD4, pinD5, pinD6, pinD7);

Encoder VEnc(pinEncV1, pinEncV2);
Encoder IEnc(pinEncI1, pinEncI2);

void getCurrentEncPositions()
{
    curEncPositions[0] = VEnc.read();
    curEncPositions[1] = IEnc.read();    

    for (int i = 0 ; i < 2 ; i++) {
        if (curEncPositions[i] != oldEncPositions[i]) 
        {
            if (curEncPositions[i] > oldEncPositions[i]) {
                encoderValues[i] += DACSetSteps[i];

                if (encoderValues[i] > 4095) encoderValues[i] = 4095;
            } 
            else {
                encoderValues[i] -= DACSetSteps[i];

                if (encoderValues[i] < 0) encoderValues[i] = 0;

            }

            oldEncPositions[i] = curEncPositions[i];
        }    
    }

    setDACOutput(pinCSV, encoderValues[0], 2);
    setDACOutput(pinCSI, encoderValues[1], 1);
}

void setDACOutput(int csPin, unsigned int val, int gain)
{
    byte g = gain == 1 ? 0x30 : 0x10;

    byte lowByte = val & 0xff;
    byte highByte = ((val >> 8) & 0xff) | g;

    digitalWrite(csPin, LOW);
    SPI.transfer(highByte);
    SPI.transfer(lowByte);
    digitalWrite(csPin, HIGH);  
}

void displayStatus()
{
    lcd.clear();

    //cal: 0.062 - 30.48
    float voltage = 4.096 * encoderValues[0] / 4096.0 * 8.0;
    //cal
    voltage = voltage / 1.054 + 0.068;
    
    lcd.print("V Set = ");
    lcd.print(voltage, 2);
    lcd.print(" V");

    float current = 2.048 * encoderValues[1] / 4096.0 * 5.0;
    lcd.setCursor(0, 1);
    lcd.print("I Set = ");
    lcd.print(current, 2);
    lcd.print(" A");        
}

void setup() {
    for (int i = 0 ; i < 2 ; i++) {
        pinMode(buttonPins[i], INPUT);
        digitalWrite(buttonPins[i], HIGH);
    }

    lcd.begin(16, 2);

    SPI.begin();  
    SPI.setClockDivider(SPI_CLOCK_DIV2);

    pinMode(pinCSV, OUTPUT);
    digitalWrite(pinCSV, HIGH);
    pinMode(pinCSI, OUTPUT);
    digitalWrite(pinCSI, HIGH);

    displayStatus();
}

void loop() {
    getCurrentEncPositions();

    for (int i = 0; i < 2; i++) {
        buttonReadings[i] = digitalRead(buttonPins[i]);

        if (buttonReadings[i] != lastButtonStates[i]) lastDebounceTime[i] = millis();

        if (millis() - lastDebounceTime[i] > 50) {
            if (currentButtonStates[i] != buttonReadings[i]) {
                currentButtonStates[i] = buttonReadings[i];

                if (currentButtonStates[i] == LOW) {
                    DACSetSteps[i] *= 10;

                    if (DACSetSteps[i] > 100) DACSetSteps[i] = 1;
                }
            }
        }

        lastButtonStates[i] = buttonReadings[i];
    }

    loopCounter++;

    if (loopCounter == LOOP_MAX_COUNT) {
        displayStatus();
        loopCounter = 0;
    }
}


